#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#define MAXP 10020
typedef long long ll;
typedef unsigned long long ull;
typedef unsigned int uint;

char sieve[MAXP];
int prime[3000];
int primes;

/*  prime number generation! */

/*  OK project euler 214 03.08.2011 */
/*  OK spoj (classical) 4942 04.08.2011 */
/*  OK spoj (tutorial) 1392 04.08.2011 */
/*  OK UVa 11073 04.08.2011 */
void createsieve() {
  int i,j,q;
  memset(sieve,1,sizeof(sieve));
  q=sqrt(MAXP);
  for(sieve[0]=sieve[1]=0,i=2;i<q;i++) if(sieve[i])
    for(j=i*i;j<MAXP;j+=i) sieve[j]=0;
}

/*  OK project euler 214 03.08.2011 */
/*  OK spoj (classical) 4942 04.08.2011 */
/*  OK spoj (tutorial) 1392 04.08.2011 */
/*  OK UVa 11073 04.08.2011 */
void genprimes() {
  int i;
  for(primes=i=0;i<MAXP;i++) if(sieve[i]) prime[primes++]=i;
}

/* factorization sieve: sieve[i] is 0 if i is prime, otherwise sieve[i] is
   smallest divisor. the datatype for sieve[] must be able to hold
   sqrt(MAXP). same asymptotic runtime as normal sieve, but higher constant
   because of the branch in the inner loop, and wider datatype */
/* OK Project Euler 386 27.08.2011 */
/* OK Topcoder Inv 2002 finals (run 1) 500 04.06.2012 */
/* OK UVa 10680 0.096 seconds n<=1000000 08.06.2012 */
int sieve2[MAXP];
void fsieve() {
  int q=sqrt(MAXP),i,j;
  memset(sieve2,0,sizeof(sieve2));
  sieve2[0]=sieve2[1]=0;
  for(i=2;i<=q;i++) if(sieve2[i]<1) for(j=i*i;j<MAXP;j+=i) if(!sieve2[j]) sieve2[j]=i;
}
/* use factorization sieve to factor a number (requires n<MAXP), runs in time
   linear to the number of prime factors (including duplicates). */
/* OK Project Euler 386 27.08.2011 */
/* OK Topcoder Inv 2002 finals (run 1) 500 04.06.2012 */
/* OK UVa 10680 0.096 seconds n<=1000000 08.06.2012 */
void ffactor(int n,int *f,int *fc,int *fn) {
	*fn=0;
	while(sieve2[n]) {
		if(*fn && f[*fn-1]==sieve2[n]) fc[*fn-1]++,n/=sieve2[n];
		else f[*fn]=sieve2[n],fc[(*fn)++]=1,n/=sieve2[n];
	}
	if(n>1) {
		if(*fn && f[*fn-1]==n) fc[*fn-1]++;
		else f[*fn]=n,fc[(*fn)++]=1;
	}
}

/*  gcd! */

ull gcd(ull a,ull b) {
  ull t;
  while(b) t=a,a=b,b=t%b;
  return a;
}

/*  slightly faster gcd */
/*  OK project euler 005 04.08.2011 */
/*  OK spoj (classical) 4942 04.08.2011 */
ull binarygcd(ull a,ull b) {
  int e=0;
  ull d;
  if(!a||!b) return a|b;
  while(!((a|b)&1)) a>>=1,b>>=1,e++;
  while(!(a&1)) a>>=1;
  do {
    while(!(b&1)) b>>=1;
    if(a<b) b-=a;
    else d=a-b,a=b,b=d;
    b>>=1;
  } while(b);
  return a<<e;
}

/*  OK project euler 005 04.08.2011 */
ull lcm(ull a,ull b) {
  return a/binarygcd(a,b)*b;
}

/* find x,y such that ax + by = gcd(a,b) */
/* UVa 10090 08.06.2012 */
ll extendedeuclid(ll a,ll b,ll *lastx,ll *lasty) {
  ll x=0,y=1,t,q;
  *lastx=1; *lasty=0;
  while(b) {
    q=a/b;
    t=a,a=b,b=t%b;
    t=x,x=*lastx-q*x,*lastx=t;
    t=y,y=*lasty-q*y,*lasty=t;
  }
  return a;
}

/* nice short recursive extended euclid */
/* warning, doesn't calculate gcd(a,b), only finds x,y */
/* WARNING, not tested */
void exeuclid(ll a,ll b,ll *x,ll *y) {
  if(!b) *x=1,*y=0;
  else exeuclid(b,a%b,y,x),*y-=*x*(a/b);
}

/*  find the multiplicative inverse of a modulo mod */
/*  requires gcd(a,mod)==1, returns 0 otherwise */
/*  OK UVa 12365 12.11.2011 */
/*	OK Project Euler 381 27.04.2012 */
ll inverse(ll a,ll mod) {
  ll b=mod,x=0,y=1,t,q,lastx=1,lasty=0;
  while(b) {
    q=a/b;
    t=a,a=b,b=t%b;
    t=x,x=lastx-q*x,lastx=t;
    t=y,y=lasty-q*y,lasty=t;
  }
  return a==1?(lastx%mod+mod)%mod:0;
}

/*	solve a set of modular equations using chinese remainder theorem */
/*	n: number of equations of the form
		x = a[i] mod b[i] */
/*	OK project euler 365 20.02.2012 */
ull chinese(int n,int *a,int *b) {
	int i;
	ull N,x=0;
	for(N=1,i=0;i<n;i++) N*=b[i];
	for(i=0;i<n;i++) x=(x+a[i]*N/b[i]*inverse(N/b[i]%b[i],b[i]))%N;
	return x;
}

/*  primality testing! */

/*  trial division, can check primality for numbers up to prime[primes-1]^2 */
/*  can actually remove the i<primes check if n is at most
    prime[primes-2]^2 */
/*  change ll to int and remove (ll) if n<2^31 always holds */
/*  NB, not tested */
int isprime(ll n) {
  int i;
  if(n<MAXP) return sieve[n];
  for(i=0;i<primes && (ll)prime[i]*prime[i]<=n;i++) if(n%prime[i]==0) return 0;
  return 1;
}

/*  table used by routine that finds log base 2 a.k.a. number of highest
    set bit */
/*  NB, needed for powmod, and hence also for witness, millerrabin,
    ullmillerrabin,factormain */
/*  NB, initlogtable must be called! */
char logtable256[256];
/*  init routine which must be called */
void initlogtable() {
  int i;
  logtable256[0]=-1;
  logtable256[1]=0;
  for(i=2;i<256;i++) logtable256[i]=1+logtable256[i>>1];
}
/*  find highest set bit of unsigned 32-bit number n and put result in r.
    t,tt are temp variables in 32, t in 64 */
#define LOG2_32(n,r,t,tt) if((tt=n>>16)) r=(t=tt>>8)?24+logtable256[t]:\
  16+logtable256[tt]; else r=(t=n>>8)?8+logtable256[t]:logtable256[n];
#define LOG2_64(n,r,t) if((t=n>>56)) r=56+logtable256[t];\
  else if((t=n>>48)) r=48+logtable256[t];\
  else if((t=n>>40)) r=40+logtable256[t];\
  else if((t=n>>32)) r=32+logtable256[t];\
  else if((t=n>>24)) r=24+logtable256[t];\
  else if((t=n>>16)) r=16+logtable256[t];\
  else if((t=n>>8)) r=8+logtable256[t];\
  else r=logtable256[n];

/*  need powmod for witness */
/*  OK UVa 10956 31.08.2011 */
/*  NOT OK YET spoj (classical) 4942 04.08.2011 */
uint powmod(uint n,uint k,uint mod) {
  int i,j;
  uint v=n,ans=1,t,tt;
  if(!k) return 1;
  /*  TODO rewrite this drivel:
  goto start;
  while(k>0) {
    v=(ull)v*v%mod;
  start:
    if(k&1) ans=(ull)ans*v%mod;
    k>>=1;
  }
  */
  /*  find topmost set bit */
  LOG2_32(k,i,t,tt)
/*  alternatively:
  i=31;
  while(i>=0 && !(n&(1<<i))) --i;
*/
  for(j=0;j<=i;j++) {
    if(k&(1U<<j)) ans=(ull)ans*v%mod;
    v=(ull)v*v%mod;
  }
  return ans;
}

/*  subroutine for miller-rabin, only works for odd n */
/*  OK UVa 10956 04.08.2011 */
/*  OK spoj (classical) 4942 04.08.2011 */
int witness(uint n,uint a) {
  int s=1,r;
  uint d=(n-1)>>1,x;
  while(!(d&1)) s++,d>>=1;
  x=powmod(a,d,n);
  if(x==1 || x==n-1) return 1;
  for(r=1;r<s;r++) {
    x=(ull)x*x%n;
    if(x==1) return 0;
    if(x==n-1) return 1;
  }
  return 0;
}

/*  deterministic miller-rabin, return 1 if n is prime */
/*  OK UVa 10956 04.08.2011 */
/*  OK spoj (classical) 4942 04.08.2011 */
int millerrabin(uint n) {
  if(n<4) return n>1;
	if(!(n&1)) return 0;
  if(n<1373653) return witness(n,2) && witness(n,3);
  if(n<9080191) return witness(n,31) && witness(n,73);
  return witness(n,2) && witness(n,7) && witness(n,61);
}

/*  needed for ullwitness */
/*  OK spoj (classical) 4942 04.08.2011 */
/*  (however, not tested for values very close to maxull) */
ull ullmulmod(ull a,ull b,ull mod) {
  ull r=0,t;
  if(a>=mod) a%=mod;
  if(b>=mod) b%=mod;
  if(a>b) t=a,a=b,b=t;
  while(b) {
    if(b&1) {
      r+=a;
      if(r>=mod || r<a) r-=mod;
    }
    a=(a<<1)%mod;
    b>>=1;
  }
  return r;
}


/*  OK spoj (classical) 4942 04.08.2011 */
ull ullpowmod(ull n,ull k,ull mod) {
  int i,j;
  ull v=n,ans=1;
  uint t;
  if(!k) return 1;
  /*  find topmost set bit */
  LOG2_64(k,i,t)
  for(j=0;j<=i;j++) {
    if(k&(1ULL<<j)) ans=ullmulmod(ans,v,mod);
    v=ullmulmod(v,v,mod);
  }
  return ans;
}

/*  subroutine for miller-rabin, only works for odd n */
/*  OK spoj (classical) 4942 04.08.2011 */
int ullwitness(ull n,ull a) {
  int s=1,r;
  ull d=(n-1)>>1,x;
  while(!(d&1)) s++,d>>=1;
  x=ullpowmod(a,d,n);
  if(x==1 || x==n-1) return 1;
  for(r=1;r<s;r++) {
    x=ullmulmod(x,x,n);
    if(x==1) return 0;
    if(x==n-1) return 1;
  }
  return 0;
}

/* deterministic miller-rabin for 64-bit numbers */
/* OK spoj (classical) 4942 04.08.2011 */
/* OK Project EUler 387 04.07.2012 */
int ullmillerrabin(ull n) {
  if(n<4294967295LU) return millerrabin(n);
	if(!(n&1)) return 0;
  if(n<4759123141LL)
    return ullwitness(n,2) && ullwitness(n,7) && ullwitness(n,61);
  if(n<2152302898747LL)
    return ullwitness(n,2) && ullwitness(n,3) && ullwitness(n,5)
      && ullwitness(n,7) && ullwitness(n,11);
  if(n<3474749660383LL)
    return ullwitness(n,2) && ullwitness(n,3) && ullwitness(n,5)
      && ullwitness(n,7) && ullwitness(n,11) && ullwitness(n,13);
  if(n<341550071728321LL)
    return ullwitness(n,2) && ullwitness(n,3) && ullwitness(n,5)
      && ullwitness(n,7) && ullwitness(n,11) && ullwitness(n,13)
      && ullwitness(n,17);
  return ullwitness(n,2) && ullwitness(n,325) && ullwitness(n,9375)
    && ullwitness(n,28178) && ullwitness(n,450775) && ullwitness(n,9780504)
    && ullwitness(n,1795265022);
}

/*  TODO solovay-strassen */
/*  TODO lucas-lehmer */
/*  TODO use algorithm from PRIMES in P */

/*  factorization */

/*  uses prime[] */
/*  write out the factors to f, the count of each f to fc, the
    number of factors to fn */
/*  OK project euler 003 09.06.2011 */
/*  OK spoj (tutorial) 1392 04.08.2011 */
/*  OK UVa 11073 04.08.2011 */
/* OK UVa 12465 n<=32000 06.06.2012 */
void factor(ull n,ull *f,int *fc,int *fn) {
  int i;
  for(*fn=i=0;i<primes && (ull)prime[i]*prime[i]<=n;i++) if(n%prime[i]<1) {
    f[*fn]=prime[i];
    fc[*fn]=1;
    n/=prime[i];
    while(n%prime[i]<1) n/=prime[i],fc[*fn]++;
    (*fn)++;
  }
  if(n>1) f[*fn]=n,fc[(*fn)++]=1;
}

/*  warning, only call on composite numbers */
/*  OK spoj (classical) 4942 04.08.2011 */
/*  OK UVa 11476 04.08.2011 */
ull pollardrho(ull n,ull c) {
  /*  number of iterations: 1.177*sqrt(sqrt(n)) is good
      source http://users.telenet.be/janneli/jan/factorization/pollard_rho.html
      TODO proof? */
  int max=1.179*sqrt(sqrt(n)),range=1,j,k;
  ull x1=2,x2=4+c,product=1,temp,g;
  for(j=0;j<max;) {
    for(k=0;k<range;k++) {
      x2=ullmulmod(x2,x2,n);
      x2+=c;
      if(x2>=n) x2-=n;
      temp=x1>x2?x1-x2:x2-x1;
      product=ullmulmod(product,temp,n);
      j++;
      if(!(j&7)) {
        g=binarygcd(product,n);
        if(g==n) return 0;
        else if(g>1) return g;
        product=1;
      }
    }
    x1=x2;
    range<<=1;
    for(k=0;k<range;k++) {
      x2=ullmulmod(x2,x2,n);
      x2+=c;
      if(x2>=n) x2-=n;
    }
  }
  return 0;
}

/*  requires primes up to MAXPRIME */
/*  OK spoj (classical) 4942 0.09 seconds 04.08.2011 */
/*  OK UVa 11476 9.600 seconds 04.08.2011 */
#define MAXPRIME 1000
void factormain(ull n,ull *f,int *fc,int *fn,int small) {
  int i,fc1[2][64],fn1[2],j,k,done;
  ull v,f1[2][64];
  *fn=0;
  if(small) {
    /*  factor out small factors */
    for(i=0;prime[i]<MAXPRIME && (ull)prime[i]*prime[i]<=n;i++) if(n%prime[i]==0) {
      n/=prime[i];
      f[*fn]=prime[i];
      fc[*fn]=1;
      while(n%prime[i]==0) fc[*fn]++,n/=prime[i];
      (*fn)++;
    }
    if(n<2) return;
    if(n<(ull)MAXPRIME*MAXPRIME) { f[*fn]=n; fc[(*fn)++]=1; return; }
  }
  if(ullmillerrabin(n)) { fc[*fn]=1; f[(*fn)++]=n; return; }
  for(i=1;!(v=pollardrho(n,i));i++);
  factormain(v,f1[0],fc1[0],&fn1[0],0);
  factormain(n/v,f1[1],fc1[1],&fn1[1],0);
  for(i=0;i<2;i++) for(j=0;j<fn1[i];j++) {
    for(k=0;k<*fn;k++) if(f[k]==f1[i][j]) {
      fc[k]+=fc1[i][j];
      goto done;
    }
    f[*fn]=f1[i][j];
    fc[(*fn)++]=fc1[i][j];
  done:;
  }
  do {
    done=1;
    for(i=0;i<*fn-1;i++) if(f[i]>f[i+1]) {
      v=f[i],f[i]=f[i+1],f[i+1]=v;
      k=fc[i],fc[i]=fc[i+1],fc[i+1]=k;
      done=0;
    }
  } while(!done);
}

int main(int argc,char **argv) {
	ull n;
	ull f[100];
	int fc[100],fn,i;
	if(argc<2) {
		puts("factor v1.0 by stubbscroll");
		puts("usage: FACTOR n");
		return 0;
	}
	if(sscanf(argv[1],"%I64u",&n)!=1) { puts("expected a number"); return 0; }
	if(n<2) { puts("expected a larger number"); return 0; }
	createsieve();
	genprimes();
	initlogtable();
	factormain(n,f,fc,&fn,MAXP-20);
	for(i=0;i<fn;i++) {
		printf("%I64u",f[i]);
		if(fc[i]>1) printf("^%d",fc[i]);
		if(i<fn-1) printf(" * ");
		else putchar('\n');
	}
	return 0;
}
